/*
*******************************************************************************
*  Copyright (c) 1996 Martin Poole                            *
*******************************************************************************
**
** WARNING !!  WARNING !! WARNING !! WARNING !! WARNING !! WARNING !!
**
** Any changes to be made to this file should first be checked with
** mplib1 source control for library integrity.
**
** mplib1 source control can be reached at  mplib1@quatermass.co.uk
** 
*
*  System         :
*  Subsystem      :
*  Module         :
*  $Source$
*  $Author$
*  $Date$
*  $Revision$
*  Purpose        :
*
*******************************************************************************
*
* $Log$
*
*******************************************************************************
*/

#ident "$Header$"

/* ------------------------------------------------------------------
    Include files
   ------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#include <gdbm.h>

#include <mplib1/dl_list.h>
#include <mplib1/fprintfile.h>
#include <mplib1/cfg_file.h>

#include <mplib1/gdbm_util.h>

/* ------------------------------------------------------------------
    defines
   ------------------------------------------------------------------ */

struct walk_det
{
FILE		*fh;
GDBM_FILE	gfile;
int		(* dfunc)(FILE *, char *, void *);
int		do_data;
int		flags;
};


/* ------------------------------------------------------------------
    Code starts here
   ------------------------------------------------------------------ */

static void
disp_item( dl_Node_t *nptr, struct walk_det *wdp )
{
    datum skey,sdata;
    int rv;

    skey.dptr = (char *)nptr->ln_Name;
    skey.dsize = (int)strlen( nptr->ln_Name ) + 1;
    if (wdp->dfunc)
    {
	if (wdp->do_data)
	{
	    sdata = gdbm_fetch( wdp->gfile, skey );
	    rv = (*wdp->dfunc)( wdp->fh, skey.dptr, sdata.dptr );
	    if (sdata.dptr && (rv & GDBM_IDUMP_KEEP_DATA)==0)
		free(sdata.dptr);
	}
	else
	    (*wdp->dfunc)( wdp->fh, skey.dptr, NULL );
    }
    else
	fprintf( wdp->fh, "Key: %s\n", skey.dptr );
    return;
}

static void
free_item( dl_Node_t *nptr )
{
    /* Free the string and the node */
    free( (char *)nptr->ln_Name );
    free( nptr );
    return;
}

static int
add_to_list( dl_List_t *lp, char *key )
{
    dl_Node_t *nptr;

    nptr = malloc( NODE_SIZE );
    if (nptr)
    {
	dl_Init_Node( nptr, key, nptr );
	dl_Add_Tail( lp, nptr );
    }
    return((nptr!=NULL));
}


int
gdbm_index_dump( FILE *fp,
		struct gdbm_help *gptr,
		int flags,
		gdbm_index_dump_t dfunc )
{
    GDBM_FILE gfile;
    datum skey,nextkey;
    int do_sort, do_hdr, do_data;
    struct walk_det walk_details;
    dl_List_t sort_list;
    dl_Node_t tnode;


    gfile = *gptr->file_ptr;
    do_hdr = (flags & GDBM_IDUMP_HDR);
    do_sort = (flags & GDBM_IDUMP_SORT);
    do_data = (flags & GDBM_IDUMP_DATA);
    dl_Init_List( &sort_list, 0 );

    walk_details.fh = fp;
    walk_details.gfile = gfile;
    walk_details.dfunc = dfunc;
    walk_details.do_data = do_data;
    walk_details.flags = flags;

    if (do_hdr)
	fprintf( fp, "   Index dump of %s\n", gptr->name );

    skey = gdbm_firstkey ( gfile );
    while ( skey.dptr )
    {
	if (do_sort)
	{
	    add_to_list( &sort_list, skey.dptr );
	}else
	{
	    tnode.ln_Name = skey.dptr;
	    disp_item( &tnode, &walk_details );
	}
	nextkey = gdbm_nextkey ( gfile, skey );
	if (do_sort==0)
	    free ( skey.dptr );
	skey = nextkey;
    };
    if (do_sort)
    {
	dl_sort_n( &sort_list );
	dl_Walk_List( &sort_list, (dl_Walk_List_t)disp_item, &walk_details );
	dl_Walk_List( &sort_list, (dl_Walk_List_t)free_item, NULL );
    }
    if (do_hdr)
	fprintf( fp, "\n" );
    return(0);
}


/* -- End of File -- */
